import React, { useCallback, useEffect, useState } from 'react';
import { ActionType } from '../../enums';
import { getValueByColumn, replaceValue } from '../../Utils/DataUtils';
import { addEscEnterKeyEffect } from '../../Utils/EffectUtils';
import { getValidationValue } from '../../Utils/Validation';
import CellEditorDataType from '../CellEditorDataType/CellEditorDataType';
import CellEditorValidationMessage from '../CellEditorValidationMessage/CellEditorValidationMessage';
const CellEditorState = (props) => {
    const { column, rowData, rowKeyField, dispatch, } = props;
    const [rowDataState, changeValue] = useState(rowData);
    const validationValue = getValidationValue(rowDataState, column);
    const onValueStateChange = (rowValue) => {
        changeValue(rowValue);
    };
    const close = useCallback(() => {
        const cell = { columnKey: column.key, rowKey: rowData[rowKeyField] };
        dispatch(ActionType.CloseEditor, { cell });
    }, [dispatch, column, rowData, rowKeyField]);
    const closeHandler = useCallback(() => {
        if (!validationValue) {
            const newValue = getValueByColumn(rowDataState, column);
            if (getValueByColumn(rowData, column) !== newValue) {
                dispatch(ActionType.ChangeRowData, { newValue: replaceValue(rowData, column, newValue) });
            }
            close();
        }
    }, [validationValue, dispatch, close, rowDataState, column, rowData]);
    useEffect(() => {
        return addEscEnterKeyEffect(close, closeHandler);
    }, [close, closeHandler]);
    const dispatchHandler = (action, actionData) => {
        if (action === ActionType.CloseEditor) {
            closeHandler();
        }
        else if (action === ActionType.ChangeRowData) {
            onValueStateChange(actionData.newValue);
        }
        else {
            dispatch(action, actionData);
        }
    };
    const stateProps = Object.assign(Object.assign({}, props), {
        dispatch: dispatchHandler,
        rowData: rowDataState,
        value: getValueByColumn(rowDataState, column),
    });
    return (React.createElement(React.Fragment, null,
        React.createElement("div", { className: `ka-cell-editor ${validationValue ? 'ka-cell-editor-validation-error' : ''}` },
            React.createElement(CellEditorDataType, Object.assign({}, stateProps)),
            validationValue && React.createElement(CellEditorValidationMessage, { message: validationValue }))));
};
export default CellEditorState;
