import * as React from 'react';
import defaultOptions from '../../defaultOptions';
import { EditingMode, FilteringMode, SortingMode } from '../../enums';
import { wrapDispatch } from '../../Utils/ActionUtils';
import { filterData, searchData } from '../../Utils/FilterUtils';
import { getExpandedGroups, getGroupedData } from '../../Utils/GroupUtils';
import { extendProps } from '../../Utils/PropsUtils';
import { sortData } from '../../Utils/SortUtils';
import { convertToColumnTypes } from '../../Utils/TypeUtils';
import FilterRow from '../FilterRow/FilterRow';
import HeadRow from '../HeadRow/HeadRow';
import TableBody from '../TableBody/TableBody';
export const Table = (props) => {
    const { childAttributes = {}, editableCells = [], editingMode = EditingMode.None, filteringMode, groups, search, selectedRows = [], sortingMode = SortingMode.None, } = props;
    let { columns, data = [], } = props;
    data = search ? searchData(columns, data, search) : data;
    data = convertToColumnTypes(data, columns);
    data = filterData(data, columns);
    data = sortData(columns, data);
    let groupColumnsCount = 0;
    let groupedColumns = [];
    if (groups) {
        groupColumnsCount = groups.length;
        groupedColumns = columns.filter((c) => groups.some((g) => g.columnKey === c.key));
        columns = columns.filter((c) => !groups.some((g) => g.columnKey === c.key));
    }
    let { groupsExpanded } = props;
    const groupedData = groups ? getGroupedData(data, groups, groupedColumns, groupsExpanded) : data;
    if (groups && !groupsExpanded) {
        groupsExpanded = getExpandedGroups(groupedData);
    }
    const theadRef = React.useRef(null);
    const dispatch = wrapDispatch(Object.assign(Object.assign({}, props), { groupsExpanded }), theadRef);
    const componentProps = {
        className: defaultOptions.css.table,
    };
    const tableProps = extendProps(componentProps, props, childAttributes.table, dispatch);
    return (React.createElement("div", { className: 'ka' },
        React.createElement("table", Object.assign({}, tableProps),
            React.createElement("thead", { className: defaultOptions.css.thead, ref: theadRef },
                React.createElement(HeadRow, { groupColumnsCount: groupColumnsCount, columns: columns, dispatch: dispatch, sortingMode: sortingMode }),
                filteringMode === FilteringMode.FilterRow &&
                    (React.createElement(FilterRow, { columns: columns, dispatch: dispatch, groupColumnsCount: groupColumnsCount }))),
            React.createElement(TableBody, Object.assign({}, props, { childAttributes: childAttributes, columns: columns, data: groupedData, dispatch: dispatch, editableCells: editableCells, editingMode: editingMode, groupColumnsCount: groupColumnsCount, groupedColumns: groupedColumns, groupsExpanded: groupsExpanded, selectedRows: selectedRows })))));
};
