import { getField } from './ColumnUtils';
export const getParentValue = (rowData, fieldParents, sameObj = false) => {
    const parentValue = fieldParents.reduce((previousValue, currentValue) => {
        const result = (previousValue && previousValue[currentValue]);
        return result !== undefined ? result : undefined;
    }, rowData);
    return parentValue ? Object.assign({}, parentValue) : undefined;
};
export const createObjByFields = (fieldParents, field, value) => {
    const parentValue = {};
    if (fieldParents.length) {
        fieldParents.reduce((previousValue, currentItem, currentIndex) => {
            const lastObj = {};
            previousValue[currentItem] = lastObj;
            if (currentIndex === (fieldParents.length - 1)) {
                lastObj[field] = value;
            }
            return lastObj;
        }, parentValue);
    }
    else {
        parentValue[field] = value;
    }
    return Object.assign({}, parentValue);
};
export const getValueByColumn = (rowData, column) => {
    return getValueByField(rowData, getField(column), column.fieldParents);
};
export const getValueByField = (rowData, field, fieldParents) => {
    if (fieldParents && fieldParents.length) {
        const parentValue = getParentValue(rowData, fieldParents);
        if (parentValue) {
            return parentValue[field];
        }
        return undefined;
    }
    else {
        return rowData[field];
    }
};
const replaceValueForField = (rowData, field, newValue, fieldParents) => {
    let result = Object.assign({}, rowData);
    if (fieldParents && fieldParents.length) {
        const parentValue = getParentValue(result, fieldParents) || {};
        parentValue[field] = newValue;
        const parentsOfParent = [...fieldParents];
        const parentFieldName = parentsOfParent.pop();
        result = replaceValueForField(result, parentFieldName, parentValue, parentsOfParent);
    }
    else {
        result[field] = newValue;
    }
    return result;
};
export const replaceValue = (rowData, column, newValue) => {
    return replaceValueForField(rowData, getField(column), newValue, column.fieldParents);
};
