import defaultOptions from '../defaultOptions';
import { DataType, FilterOperatorName } from '../enums';
import { isEmpty } from './CommonUtils';
import { getValueByColumn } from './DataUtils';
export const getRowEditableCells = (rowKeyValue, editableCells) => {
    return editableCells.filter((c) => c.rowKey === rowKeyValue);
};
export const searchData = (columns, data, searchText) => {
    return columns.reduce((initialData, c) => {
        const filterFunction = (item) => {
            if (c.search) {
                return c.search(searchText, item, c);
            }
            if (initialData.indexOf(item) >= 0) {
                return false;
            }
            const columnValue = getValueByColumn(item, c);
            if (columnValue == null) {
                return false;
            }
            return columnValue.toString().toLowerCase().includes(searchText.toLowerCase());
        };
        return initialData.concat(data.filter(filterFunction));
    }, []);
};
export const filterData = (data, columns) => {
    return columns.reduce((initialData, column) => {
        if (isEmpty(column.filterRowValue)
            && column.filterRowOperator !== FilterOperatorName.IsEmpty
            && column.filterRowOperator !== FilterOperatorName.IsNotEmpty) {
            return initialData;
        }
        const filterRowOperator = column.filterRowOperator
            || getDefaultOperatorForType(column.dataType || defaultOptions.columnDataType);
        const filterOperator = predefinedFilterOperators.find((fo) => filterRowOperator === fo.name);
        if (!filterOperator) {
            throw new Error(`'${column.filterRowOperator}' has not found in predefinedFilterOperators array, available operators: ${predefinedFilterOperators.map((o) => o.name).join(', ')}`);
        }
        const compare = filterOperator.compare;
        return initialData.filter((d) => {
            let fieldValue = getValueByColumn(d, column);
            let conditionValue = column.filterRowValue;
            if (column.dataType === DataType.Date) {
                fieldValue = fieldValue == null ? fieldValue : new Date(fieldValue).setHours(0, 0, 0, 0);
                conditionValue = conditionValue == null ? conditionValue : new Date(conditionValue).setHours(0, 0, 0, 0);
            }
            return compare(fieldValue, conditionValue);
        });
    }, data);
};
export const getDefaultOperatorForType = (type) => {
    const filterOperator = predefinedFilterOperators.find((o) => o.defaultForTypes && o.defaultForTypes.includes(type));
    return (filterOperator && filterOperator.name) || '=';
};
export const predefinedFilterOperators = [{
        compare: (fieldValue, conditionValue) => fieldValue === conditionValue,
        defaultForTypes: [DataType.Boolean, DataType.Number, DataType.Date],
        name: FilterOperatorName.Equal,
    }, {
        compare: (fieldValue, conditionValue) => fieldValue > conditionValue,
        name: FilterOperatorName.MoreThan,
    }, {
        compare: (fieldValue, conditionValue) => fieldValue < conditionValue,
        name: FilterOperatorName.LessThan,
    }, {
        compare: (fieldValue, conditionValue) => fieldValue >= conditionValue,
        name: FilterOperatorName.MoreThanOrEqual,
    }, {
        compare: (fieldValue, conditionValue) => fieldValue <= conditionValue,
        name: FilterOperatorName.LessThanOrEqual,
    }, {
        compare: (fieldValue, conditionValue) => fieldValue != null && fieldValue.toString().toLowerCase().includes(conditionValue.toString().toLowerCase()),
        defaultForTypes: [DataType.String],
        name: FilterOperatorName.Contains,
    }, {
        compare: (fieldValue) => isEmpty(fieldValue),
        name: FilterOperatorName.IsEmpty,
    }, {
        compare: (fieldValue) => !isEmpty(fieldValue),
        name: FilterOperatorName.IsNotEmpty,
    }];
