import { getValueByColumn } from './DataUtils';
const groupMark = {};
export const updateExpandedGroups = (groupsExpanded, groupKey) => {
    const newGroupsExpanded = groupsExpanded.filter((ge) => JSON.stringify(ge) !== JSON.stringify(groupKey));
    if (newGroupsExpanded.length === groupsExpanded.length) {
        newGroupsExpanded.push(groupKey);
    }
    return newGroupsExpanded;
};
export const getExpandedGroups = (groupedData) => {
    return groupedData
        .filter((g) => g.groupMark === groupMark)
        .map((g) => g.key);
};
export const getGroupedData = (data, groups, groupedColumns, groupsExpanded) => {
    const grouped = getGroupedStructure(data, groups, groupedColumns, 0, groupsExpanded);
    return convertToFlat(grouped);
};
export const convertToFlat = (grouped, key = []) => {
    let result = [];
    grouped.forEach((value, groupValue) => {
        const groupKey = [...key];
        groupKey.push(groupValue);
        result.push({ groupMark, key: groupKey, value: groupValue });
        result = [...result, ...(Array.isArray(value) ? value : convertToFlat(value, groupKey))];
    });
    return result;
};
export const getGroupedStructure = (data, groups, groupedColumns, expandedDeep = 0, groupsExpanded) => {
    groups = [...groups];
    const group = groups.shift();
    if (group) {
        const column = groupedColumns && groupedColumns.find((g) => g.key === group.columnKey);
        if (column) {
            const grouped = groupBy(data, (item) => getValueByColumn(item, column));
            grouped.forEach((value, key) => {
                const groupExpandedItems = groupsExpanded && groupsExpanded.filter((ge) => ge[expandedDeep] === key);
                const isThisGroupExpanded = !groupExpandedItems
                    || groupExpandedItems.some((ge) => ge.length === expandedDeep + 1);
                if (isThisGroupExpanded) {
                    const newStructure = getGroupedStructure(value, groups, groupedColumns, expandedDeep + 1, groupExpandedItems && groupExpandedItems.filter((ge) => ge.length > expandedDeep + 1));
                    if (newStructure) {
                        grouped.set(key, newStructure);
                    }
                }
                else {
                    grouped.set(key, []);
                }
            });
            return grouped;
        }
    }
};
export const groupBy = (data, keyGetter, isEmptyValue = false) => {
    const map = new Map();
    data.forEach((item) => {
        const key = keyGetter(item);
        if (isEmptyValue) {
            map.set(key, []);
        }
        else {
            const collection = map.get(key);
            if (!collection) {
                map.set(key, [item]);
            }
            else {
                collection.push(item);
            }
        }
    });
    return map;
};
export const getGroupMark = () => groupMark;
export const getGroupText = (value, column) => {
    const format = column && column.format;
    return format ? format(value) : `${column && column.title && (column.title + ': ')}${value}`;
};
