import { SortDirection } from '../enums';
import { getValueByColumn } from './DataUtils';
export const sortData = (columns, data) => {
    const sortedColumn = columns.find((column) => column.sortDirection);
    if (!sortedColumn) {
        return data;
    }
    const sortFunc = sortedColumn.sortDirection === SortDirection.Ascend
        ? ascendSort(sortedColumn) : descendSort(sortedColumn);
    const newData = [...data].sort(sortFunc);
    return newData;
};
export const ascendSort = (sortedColumn) => {
    return (a, b) => {
        const aValue = getValueByColumn(a, sortedColumn);
        const bValue = getValueByColumn(b, sortedColumn);
        if (aValue === bValue) {
            return 0;
        }
        else if (aValue == null) {
            return -1;
        }
        else if (bValue == null) {
            return 1;
        }
        return aValue < bValue ? -1 : 1;
    };
};
export const descendSort = (sortedColumn) => {
    return (a, b) => {
        const aValue = getValueByColumn(a, sortedColumn);
        const bValue = getValueByColumn(b, sortedColumn);
        if (aValue === bValue) {
            return 0;
        }
        else if (aValue == null) {
            return 1;
        }
        else if (bValue == null) {
            return -1;
        }
        return aValue > bValue ? -1 : 1;
    };
};
